@@ -1,5 +1,5 @@
 /*
- * ScanSSH - simple SSH version scanner V1.2
+ * ScanSSH - simple SSH version scanner V1.3
  *
  * Copyright 2000 Niels Provos <provos@citi.umich.edu>
  * All rights reserved.
@@ -75,7 +75,7 @@ int alarmed;
 #define CONNECTWAIT	5
 #define SYNWAIT		1
 #define SYNRETRIES	5
-#define MAXBUF		255
+#define MAXBUF		2048
 #define MAXSYNQUEUESZ	256
 #define MAXBURST	64
 
@@ -333,6 +333,7 @@ printres(struct argument *exp, char *result)
 	char ntop[NI_MAXHOST];
 	ipv4toa(ntop, sizeof(ntop), &exp->a_ipv4);
 	fprintf(stdout, "%s %s\n", ntop, result);
+	fflush(stdout);
 }
 
 void
@@ -381,17 +382,23 @@ scanips(struct argument *args, u_int32_t entries)
 
 		/* Read back commands */
 		for (i = 0; i < MAX_PROCESSES; i++) {
+			int count;
 			if (results[i] < 0 || !FD_ISSET(results[i], readset))
 				continue;
 
 			res = -1;
-			while (res == -1) {
-				res = read(results[i], buf, sizeof(buf));
+			count = 0;
+			while (res != 0 && count < sizeof(buf)) {
+				res = read(results[i], buf + count,
+					   sizeof(buf) - count);
 				if (res == -1 &&
 				    (errno != EINTR && errno != EAGAIN))
 					break;
-				if (res != -1 && buf[res - 1] == '\0')
-					break;
+				if (res > 0) {
+					count += res;
+					if (buf[count - 1] == '\0')
+						break;
+				}
 			}
 			if (res == 0) {
 				/* Child terminated */
@@ -401,14 +408,13 @@ scanips(struct argument *args, u_int32_t entries)
 					close(commands[i]);
 					commands[i] = -1;
 				}
-				continue;
 			} else if(res == -1)
 				err(1, "read");
 			p = buf;
-			while (res > 0) {
+			while (count > 0) {
 				fprintf(stdout, "%s\n", p);
 				fflush(stdout);
-				res -= strlen(p) + 1;
+				count -= strlen(p) + 1;
 				p += strlen(p) + 1;
 				counts--;
 			}
@@ -624,7 +630,7 @@ populate(struct argument **args, u_int32_t *entries, sa_family_t type,
 		n = 0;
 		for (ipv4next = ipv4addr;
 		     ipv4next.s_addr <= ipv4end.s_addr &&
-			     ipv4next.s_addr != INADDR_ANY;
+			     !(ipv4next.s_addr == INADDR_ANY && n > 0);
 		     ipv4next.s_addr++, n++) {
 			ipv4next = exclude(ipv4next, &excludequeue);
 			if (ipv4next.s_addr > ipv4end.s_addr)
@@ -637,10 +643,12 @@ populate(struct argument **args, u_int32_t *entries, sa_family_t type,
 		n += *entries;
 	} else {
 		struct in_addr tmp;
+		u_int32_t numbers;
 
 		rndsboxinit();
 		n = 0;
-		for (i = 0; i < dorandom; i++) {
+		numbers = 1 << (32 - bits);
+		for (i = 0; (n < dorandom) && (i < numbers); i++) {
 			ipv4next = rndgetaddr(&ipv4addr, bits, i);
 			tmp = exclude(ipv4next, &rndexclqueue);
 			if (ipv4next.s_addr != tmp.s_addr)
@@ -656,6 +664,8 @@ populate(struct argument **args, u_int32_t *entries, sa_family_t type,
 		if (n != dorandom)
 			fprintf(stderr, "Excluding %d addresses\n",
 				dorandom - n);
+		if (i > n)
+			fprintf(stderr, "Skipped %d addresses\n", i - n);
 		n += *entries;
 	}
 	
@@ -685,7 +695,10 @@ populate(struct argument **args, u_int32_t *entries, sa_family_t type,
 	} else {
 		int count = 0;
 		struct in_addr tmp;
-		for (i = 0; i < dorandom; i++) {
+		u_int32_t numbers;
+
+		numbers = 1 << (32 - bits);
+		for (i = 0; (count < dorandom) && (i < numbers); i++) {
 			ipv4next = rndgetaddr(&ipv4addr, bits, i);
 			tmp = exclude(ipv4next, &rndexclqueue);
 			if (ipv4next.s_addr != tmp.s_addr)
